@@ -2,7 +2,7 @@
 
 // fixg2sxd - a utility to convert fig to sxd format
 
-// Copyright (C) 2003,2004 Alexander Bürger, acfb@users.sourceforge.net
+// Copyright (C) 2003-2006 Alexander Bürger, acfb@users.sourceforge.net
 
 // This program is free software; you can redistribute it and/or
 // modify it under the terms of the GNU General Public License as
@@ -27,6 +27,7 @@
 #include <string>
 #include <iostream>
 #include <fstream>
+#include <sstream>
 #include <vector>
 
 using namespace std;
@@ -258,42 +259,47 @@ static void write_file( ostream& sxdfile )
 
 static void read_file( istream& figfile )
 {
-    char linebuf[80];
+    string linebuf;
 
-    // check if it's a fig file
-    figfile.getline( linebuf, sizeof(linebuf) );
-    if( strncasecmp( linebuf, "#FIG 3.2", 8 ) != 0 ) {
+    // check if it's a fig file; first line is a comment, but we
+    // should not skip it
+    getline( figfile, linebuf );
+    if( linebuf.compare( 0, 8, "#FIG 3.2" ) != 0 ) {
         cerr << "linebuf =>" << linebuf << "<=" << endl;
         fail( "Not a figfile or unknown fig format (only 3.2 is known)." );
     }
 
     // read orientation
-    figfile.getline( linebuf, sizeof(linebuf) );
-    if( strcasecmp( linebuf, "Landscape" ) == 0 ) {
+    skip_comment( figfile );
+    getline( figfile, linebuf );
+    if( linebuf == "Landscape" ) {
         orientation = Landscape;
-    } else if( strcasecmp( linebuf, "Portrait" ) != 0 ) {
+    } else if( linebuf != "Portrait" ) {
         fail( "Bad orientation." );
     }
 
     // read justification
     enum { Center, FlushLeft } justification = Center;
-    figfile.getline( linebuf, sizeof(linebuf) );
-    if( strcasecmp( linebuf, "Flush left" ) == 0 ) {
+    skip_comment( figfile );
+    getline( figfile, linebuf );
+    if( linebuf == "Flush left" ) {
         justification = FlushLeft;
-    } else if( strcasecmp( linebuf, "Center" ) != 0 ) {
+    } else if( linebuf != "Center" ) {
         fail( "Bad justification." );
     }
 
     // read units
     enum { Metric, Inches } units = Metric;
-    figfile.getline( linebuf, sizeof(linebuf) );
-    if( strcasecmp( linebuf, "Inches" ) == 0 ) {
+    skip_comment( figfile );
+    getline( figfile, linebuf );
+    if( linebuf == "Inches" ) {
         units = Inches;
-    } else if( strcasecmp( linebuf, "Metric" ) != 0 ) {
+    } else if( linebuf != "Metric" ) {
         fail( "Bad units." );
     }
 
     // read papersize
+    skip_comment( figfile );
     string papersize_name;
     figfile >> papersize_name;
     for( papersize=0; papersize<15; ++papersize ) {
@@ -304,20 +310,23 @@ static void read_file( istream& figfile )
         fail( "Bad papersize." );
 
     // read magnification
+    skip_comment( figfile );
     float magnification;
     figfile >> magnification;
     figfile.ignore( 1024, '\n' );
 
     // read multipageness
+    skip_comment( figfile );
     enum { Single, Multiple } multipage = Single;
-    figfile.getline( linebuf, sizeof(linebuf) );
-    if( strcasecmp( linebuf, "Multiple" ) == 0 ) {
+    getline( figfile, linebuf );
+    if( linebuf == "Multiple" ) {
         multipage = Multiple;
-    } else if( strcasecmp( linebuf, "Single" ) != 0 ) {
+    } else if( linebuf != "Single" ) {
         fail( "Bad multipageness." );
     }
 
     // read transparent color
+    skip_comment( figfile );
     int transparentcolor = -5;
     figfile >> transparentcolor;
     if( transparentcolor < -3 || transparentcolor > 543 )
@@ -379,9 +388,10 @@ static void read_file( istream& figfile )
             c->read( figfile );
             xfigobjects.push_back( c );
             break; }
-        default:
-            cerr << "Unknown object code: " << object_code << endl;
-            exit( -1 );
+        default: {
+            ostringstream err;
+            err << "Unknown object code: " << object_code;
+            throw err.str(); }
         }
     }
 }
@@ -389,7 +399,7 @@ static void read_file( istream& figfile )
 static void usage( const char* cmdname, bool err=true )
 {
     ostream& out = (err ? cerr : cout);
-    out << cmdname << " [-l(ine)w(idth)1 l] figfile [sxdfile]" << endl << endl
+    out << cmdname << " [-w] [-l(ine)w(idth)1 l] figfile [sxdfile]" << endl << endl
         << "Using `-' as figfile makes the program read from stdin." << endl
         << endl
         << "Omitting sxdfile when figfile ends with .fig or .xfig, makes"<<endl
@@ -397,7 +407,9 @@ static void usage( const char* cmdname, bool err=true )
         << endl << endl
         << "With -linewidth1 or -lw1 the with of lines with thickness 1 in xfig" <<endl
         << "can be set (in cm), e.g. to 0 to have fine lines." << endl << endl
-        << "This is fig2sxd version 0.13. (C) 2003,2004 Alexander Bürger."
+        << "With -w files with out-of-specification values will be accepted," <<endl
+        << "but a warning will be printed and the bad values sanitized." << endl << endl
+        << "This is fig2sxd version 0.16. (C) 2003-2006 Alexander Bürger."
         << endl;
 }
 
@@ -418,7 +430,7 @@ static bool option( const char* option, const char* &value,
     return found;
 }
 
-#if 0
+#if 1
 static bool option0( const char* option, int &argc, char** argv )
 {
     bool found = false;
@@ -435,7 +447,9 @@ static bool option0( const char* option, int &argc, char** argv )
 }
 #endif
 
-int main( int argc, char* argv[] )
+extern bool out_of_range_error;
+
+int try_catched_main( int argc, char* argv[] )
 {
     const char* tmp;
     if( option( "-linewidth1", tmp, argc, argv ) | option( "-lw1", tmp, argc, argv ) ) {
@@ -444,6 +458,10 @@ int main( int argc, char* argv[] )
             fail( "linewith must be >= 0" );
         LineFillStyle::linewith1 = w;
     }
+    if( option0( "-w", argc, argv ) ) {
+        out_of_range_error = false;
+        cerr << "Out of range values will be sanitized." << endl;
+    }
 
     bool dotfig=false, dotxfig=false;
     string figfilename;
@@ -479,4 +497,19 @@ int main( int argc, char* argv[] )
         sxd = argv[2];
     ofstream sxdfile( sxd.c_str(), ios::binary );
     write_file( sxdfile );
+    return 0;
+}
+
+int main( int argc, char* argv[] )
+{
+    try {
+        return try_catched_main( argc, argv );
+    } catch( string msg ) {
+        cerr << msg << endl << endl
+             << "If you think this is message is caused by a problem in fig2sxd," << endl
+             << "please contact the author(s) via http://fig2sxd.sourceforge.net/."  << endl
+             << "Thank you." << endl << endl
+             << "To get your file converted anyway, you could try the '-w' option." << endl;
+        return -1;
+    }
 }
