@@ -2,7 +2,7 @@
 
 // fixg2sxd - a utility to convert fig to sxd format
 
-// Copyright (C) 2003,2004 Alexander Bürger, acfb@users.sourceforge.net
+// Copyright (C) 2003-2006 Alexander Bürger, acfb@users.sourceforge.net
 
 // This program is free software; you can redistribute it and/or
 // modify it under the terms of the GNU General Public License as
@@ -20,9 +20,10 @@
 
 #include "xfigobjects.h"
 
-#include "xmlwrite.h"
+#include "check.h"
 #include "misc.h"
 #include "vector2.h"
+#include "xmlwrite.h"
 
 #include <cmath>
 
@@ -38,19 +39,24 @@ Spline::~Spline()
 istream& Spline::read( istream& figfile )
 {
     LineFillStyle lfstmp;
-    figfile >> sub_type
-            >> lfstmp.line_style
-            >> lfstmp.line_thickness
-            >> lfstmp.pen_color
-            >> lfstmp.fill_color
-            >> depth
-            >> lfstmp.pen_style
-            >> lfstmp.area_fill
-            >> lfstmp.style_val
-            >> cap_style
+    lfstmp.read( figfile, sub_type, depth );
+    figfile >> cap_style
             >> forward_arrow
             >> backward_arrow
             >> npoints;
+
+    keep_range( sub_type,       "spline sub_type", 0, 5 );
+    if( (sub_type&1)==0 ) // cap_style is used only for open splines
+        keep_range( cap_style,  "spline(open) cap_style", 0, 2 );
+    keep_range( forward_arrow,  "spline forward_arrow",  0, 1 );
+    keep_range( backward_arrow, "spline backward_arrow", 0, 1 );
+    if( npoints<2 || npoints>65536 ) {
+        ostringstream err;
+        err << "Bad number of spline points " << npoints
+            << " (accepted range = [2,65536]).";
+        throw err.str();
+    }
+
     if( forward_arrow != 0 )
         lfstmp.read_arrow( figfile, true );
     if( backward_arrow != 0 )
